"""Dismantle is a Python package / plugin / extension manager."""
import logging

from dismantle.__version__ import __version__  # noqa: F401

logging.getLogger(__name__).addHandler(logging.NullHandler())



import abc
import shutil
import tarfile
import zipfile
from pathlib import Path
from typing import Union


class PackageFormat(metaclass=abc.ABCMeta):
    """Base class for packet formats."""

    @staticmethod
    @abc.abstractmethod
    def grasps(path: Union[str, Path]) -> bool:
        """Check if format can be processed."""
        ...

    @staticmethod
    @abc.abstractmethod
    def verify(signature: str) -> bool:
        """Verify a packages hash with the provided signature."""
        ...

    @staticmethod
    @abc.abstractmethod
    def extract(src: Union[str, Path], dest: Union[str, Path]) -> bool:
        """Verify a packages hash with the provided signature."""
        ...


class DirectoryPackageFormat(PackageFormat):
    """A package format using a directory to hold files."""

    @staticmethod
    def grasps(path: Union[str, Path]) -> bool:
        """Check if dir on the local filesystem has been provided."""
        path = str(path)[7:] if str(path)[:7] == 'file://' else path
        try:
            return Path(str(path)).is_dir()
        except OSError:
            return False

    @staticmethod
    def extract(src: Union[str, Path], dest: Union[str, Path]) -> None:
        """Use the formatter to process any movement related actions."""
        src = str(src)[7:] if str(src)[:7] == 'file://' else src
        dest = str(dest)[7:] if str(dest)[:7] == 'file://' else dest
        if not DirectoryPackageFormat.grasps(src):
            message = 'formatter only supports directories'
            raise ValueError(message)
        if dest != src:
            shutil.rmtree(dest, ignore_errors=True)
            shutil.copytree(
                src,
                dest,
                ignore=shutil.ignore_patterns('.git', '__pycache__')
            )


class ZipPackageFormat(PackageFormat):
    """A package format compressed as a zip file."""

    @staticmethod
    def grasps(path: Union[str, Path]) -> bool:
        """Check if dir on the local filesystem has been provided."""
        path = str(path)[7:] if str(path)[:7] == 'file://' else path
        zip_path = Path(path)
        if zip_path.suffix != '.zip':
            return False
        return True

    @staticmethod
    def extract(src: Union[str, Path], dest: Union[str, Path]) -> None:
        """Extract the zipfile to the cache location."""
        src = str(src)[7:] if str(src)[:7] == 'file://' else src
        dest = str(dest)[7:] if str(dest)[:7] == 'file://' else dest
        src = Path(src)
        dest_path = Path(dest)

        if not ZipPackageFormat.grasps(src):
            message = 'formatter only supports zip files'
            raise ValueError(message)
        if not src.is_file() or not zipfile.is_zipfile(src):
            message = 'invalid zip file'
            raise ValueError(message)
        with zipfile.ZipFile(src, 'r') as zip_ref:
            zip_ref.extractall(dest_path)


class TarPackageFormat(PackageFormat):
    """A package format using a compressed tar file."""

    @staticmethod
    def grasps(path: Union[str, Path]) -> bool:
        """Check if dir on the local filesystem has been provided."""
        path = str(path)[7:] if str(path)[:7] == 'file://' else path
        tar_path = Path(path)
        suffixes = ''.join(tar_path.suffixes)
        if suffixes != '.tar':
            return False
        return True

    @staticmethod
    def extract(src: Union[str, Path], dest: Union[str, Path]) -> None:
        """Extract the tarfile to the cache location."""
        src = str(src)[7:] if str(src)[:7] == 'file://' else src
        dest = str(dest)[7:] if str(dest)[:7] == 'file://' else dest
        src_path = Path(src)
        dest_path = Path(dest)

        if not TarPackageFormat.grasps(src_path):
            message = 'formatter only supports tar files'
            raise ValueError(message)
        if not src_path.is_file() or not tarfile.is_tarfile(src_path):
            message = 'invalid tar file'
            raise ValueError(message)
        with tarfile.open(src_path, 'r') as tar_ref:
            tar_ref.extractall(dest_path)


class TgzPackageFormat(PackageFormat):
    """A package format using a tgz file compression."""

    @staticmethod
    def grasps(path: Union[str, Path]) -> bool:
        """Check if dir on the local filesystem has been provided."""
        path = str(path)[7:] if str(path)[:7] == 'file://' else path
        src_path = Path(path)
        suffixes = ''.join(src_path.suffixes)
        if suffixes not in ['.tgz', '.tar.gz']:
            return False
        return True

    @staticmethod
    def extract(src: Union[str, Path], dest: Union[str, Path]) -> None:
        """Extract the tarfile to the cache location."""
        src = str(src)[7:] if str(src)[:7] == 'file://' else src
        dest = str(dest)[7:] if str(dest)[:7] == 'file://' else dest
        src_path = Path(src)
        dest_path = Path(dest)

        if not TgzPackageFormat.grasps(src_path):
            message = 'formatter only supports tar.gz files'
            raise ValueError(message)
        if not src_path.is_file() or not tarfile.is_tarfile(src_path):
            message = 'invalid tgz file'
            raise ValueError(message)
        with tarfile.open(src_path, 'r') as tgz_ref:
            tgz_ref.extractall(dest_path)



"""Check packages."""
import argparse
import sys

from dismantle import __version__


def build_parser():
    """Build the argument for the cli."""
    parser = argparse.ArgumentParser(
        'dismantle',
        description='Dismantle package management'
    )
    return parser


def main(args=None):
    """Add entry point for python -m option."""
    args = args if args else sys.argv[1:]
    parser = build_parser()
    app = '%(prog)s version ' + __version__
    parser.add_argument('-v', '--version', action='version', version=app)
    args = parser.parse_args(args)



"""Configure PyTest."""
import shutil
from pathlib import Path

import pytest


@pytest.fixture()
def datadir(tmpdir: Path, request):
    """Provide a data directory for unit tests.

    Fixture responsible for searching a folder with the same name of
    test module and, if available, moving all contents to a temporary
    directory so tests can use them freely.
    """
    filename = Path(request.module.__file__)
    test_dir = Path(filename.parent, 'data', filename.stem)

    if test_dir.is_dir():
        shutil.rmtree(str(tmpdir), ignore_errors=True)
        shutil.copytree(test_dir, str(tmpdir))

    return tmpdir


@pytest.fixture(scope='session')
def httpserver_listen_address():
    """Use port 9090 for testing."""
    return ('127.0.0.1', 9090)



"""Simple extension to provide greetings."""
from dismantle.extension import IExtension


class GreetingExtension(IExtension):
    """A test extension to provide greetings."""

    _category = 'greeting'

    def greet(self, person: str) -> None:
        """Test method to greet a person."""
        ...



"""Creates an interface to allow all other extensions to extend."""


class IExtension:
    """Extension interface to recognise what needs to be processed."""

    _name: str = 'default'
    _category: str = 'default'

    def __init__(self) -> None:
        """Initialise with default activity set to false."""
        self._active: bool = False

    def activate(self) -> bool:
        """Activate an extension."""
        self._active = True
        return self._active

    def deactivate(self) -> bool:
        """Deactivate an extension."""
        self._active = False
        return not self._active

    @property
    def name(self) -> str:
        """Return the extensions name."""
        return self._name

    @property
    def category(self) -> str:
        """Return the extensions category."""
        return self._category

    @property
    def active(self) -> bool:
        """Return the extensions active state."""
        return self._active
